#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

/*
exec_command()で
・前処理
　・vid, nidが認識可能な単語でなければそのむね表示し終了
・area_xxxを呼び出す。非0が返されたら終了
　・その場に存在する名詞についてはexistを非0に設定
　・後処理のデフォルト動作以外を行う場合は処理を記述
・後処理
　・名詞なしの各動詞はデフォルト動作
　・アイテム扱いの名詞ならexec_command_itemを呼び、非0が返ったら終了
　・名詞が存在してexistが0ならその場に存在しない旨表示して終了
　・任意の名詞に対するデフォルト動作、あるいは一般名詞(壁)などに対する
　　デフォルト動作をして終了
　・ここまででいずれの処理にも引っ掛からなければ、「ここではできない」と表示
*/



int area_xxx(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
    *exist = 1;
    switch(nid){
    // 名詞なしの場合。基本的には移動コマンドのみだが、
    // 特殊な場面で一部のコマンドをオーバライドしたいばあいはここに追加
    case ni_invalid:
        switch(vid){
        case vi_look:
            descript_area(ai_xxx);
            /*
            場面の詳しい説明を出力。
            条件により説明が変わる場合は、たとえば以下のようにする

            if (is_flag_on(fi_xxx)){
                message("");
            }

            */
            return 1;
        case vi_forward:
        case vi_backward:
        case vi_right:
        case vi_left:
        case vi_up:
        case vi_down:
            /*
            移動可能な方向のみ記述(処理しない場合は移動できなくなる)

            move_area(ai_yyy);

            ドアを通る場合は、以下のようにしてドア通過フラグを立てる   

            enter_door(ai_yyy, fi_yyy_door_opened);

            特別な理由で移動できない場合はメッセージを出す

            message("扉は不思議な力で固く閉ざされている。");

            */
            return 1;
        default:
            break;
        }
        break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
    case ni_corridor:
        switch (vid){
        case vi_look:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
    case ni_ceiling:
    case ni_wall:
    case ni_floor:
	case ni_window:
    case ni_stairs:
        /*
        何もできない壁や床などについても、その場面に存在するなら
        0を返して終了する
        存在しない場合はcaseを書かなければ「見当たらない」になる
        当然、なんらかの動作が可能な場面では具体的に処理を書き、
        デフォルト動作をスキップするため非0を返す
        */
        break;
    case ni_door:
        /*
        ドアが複数ある場合は以下のようにしてすべての動詞を無効化する

        msg_which_door();
        return 1;

        ドアが一つしかない場合は、次の複数ドアの場合と同様に個別処理を書く
                
        */
    case ni_front_door:
    case ni_back_door:
    case ni_left_door:
    case ni_right_door:
        /*
        複数ドアがある場合(存在する方向のもののみ書く)


        続いて各動詞を処理する

        if (vid == vi_open || vid == vi_push || vid == vi_pull){
            enter_door(ai_entrance_hall, fi_hall_right_corridor_door_opened);
            return 1;
	}
	break;


        switch(vid){
        case vi_open:
        case vi_push:
        case vi_pull:
            if (nid == ni_front_door){
                enter_door(ai_yyy, fi_yyy_door_opened);
            }
            else if (nid == ni_back_door) {
                enter_door(ai_yyy, fi_yyy_door_opened);
            }
            else if (nid == ni_left_door) {
                enter_door(ai_yyy, fi_yyy_door_opened);
            }
            else if (nid == ni_right_door) {
                enter_door(ai_yyy, fi_yyy_door_opened);
            }
            return 1;
        default:
            break;
        }
        */
        break;
    case ni_xxx:
        /*
        有効な各動詞の処理を記述
        デフォルト動作で良い動詞については、書かなければ後処理で処理される

        switch(vid){
        case vi_xxx:
            return 1;
        default:
            break;
        }
        */
    default:
        /* 言及がない名詞はその場にないものとする */
        *exist = 0;
        break;
    }
    return 0;
}

// area_xxxには即座に効力を発揮するフラグ処理(机の移動など)しか記述できないため、
// 時間経過で進むイベント処理はこの関数を定義し、
// ・area.hにプロトタイプ宣言を追加
// ・area.cのareasに関数ポインタを追加
// する。
// この関数は、ユーザがコマンド入力を行う度に、そのコマンドを*処理した後で*呼び出される。
// callbackで追加入力などを処理する際には呼び出されない？
void post_xxx(int time_passed){
    // lookなど時間が経過しないコマンドが実行された場合はtime_passed==0なので、
    // 通常はなにもせずに戻る
    if (!time_passed) return;
    // area_xxxでイベント発動フラグを立てたターンでpost_xxxが実行されるので、
    // 通常はarea_xxxではstart_yyyという名前のフラグを立てておき、
    // post_xxx内でフラグyyyを立ててそのターンのメッセージを表示する。
    // 以後、何ターンかかかるイベントの時は順にフラグを立てていく
    // ・同一イベント用の一連のフラグは、ターン毎に順に立てていき、
    //   イベント終了時(or解除時)にまとめてリセットする(最初の方のフラグを途中でリセットしない)
    //   これは、イベント進行中か否かをIs(start_yyy)でチェックできるようにするためである。
    //   その代わり、一連のフラグチェックは後ろから順にif～else if～で書く必要がある。
    if (Is(zzz)){
        message("event final phase");
        die();
    }
    else if (Is(yyy)){
        message("event phase 1");
        Set(zzz);
    }
    else if (Is(start_yyy)){
        message("event start");
        Set(yyy);
    }
}
